@@ -2,23 +2,25 @@
 Keyring implementation support
 """
 
+from __future__ import annotations
+
 import os
 import abc
 import logging
 import operator
+import copy
 
-from typing import Optional
-
-import importlib_metadata as metadata
+import typing
 
+from .py312compat import metadata
 from . import credentials, errors, util
-from .util import properties
+from ._compat import properties
 
 log = logging.getLogger(__name__)
 
 
 by_priority = operator.attrgetter('priority')
-_limit = None
+_limit: typing.Optional[typing.Callable[[KeyringBackend], bool]] = None
 
 
 class KeyringBackendMeta(abc.ABCMeta):
@@ -44,8 +46,8 @@ class KeyringBackend(metaclass=KeyringBackendMeta):
     def __init__(self):
         self.set_properties_from_env()
 
-    # @abc.abstractproperty
-    def priority(cls):
+    @properties.classproperty
+    def priority(self) -> float:
         """
         Each backend class must supply a priority, a number (float or integer)
         indicating the priority of the backend relative to all other backends.
@@ -60,24 +62,25 @@ class KeyringBackend(metaclass=KeyringBackendMeta):
         As a rule of thumb, a priority between zero but less than one is
         suitable, but a priority of one or greater is recommended.
         """
+        raise NotImplementedError
 
-    @properties.ClassProperty
-    @classmethod
+    @properties.classproperty
     def viable(cls):
         with errors.ExceptionRaisedContext() as exc:
             cls.priority
         return not exc
 
     @classmethod
-    def get_viable_backends(cls):
+    def get_viable_backends(
+        cls: typing.Type[KeyringBackend],
+    ) -> filter[typing.Type[KeyringBackend]]:
         """
         Return all subclasses deemed viable.
         """
         return filter(operator.attrgetter('viable'), cls._classes)
 
-    @properties.ClassProperty
-    @classmethod
-    def name(cls):
+    @properties.classproperty
+    def name(cls) -> str:
         """
         The keyring name, suitable for display.
 
@@ -85,16 +88,16 @@ class KeyringBackend(metaclass=KeyringBackendMeta):
         """
         parent, sep, mod_name = cls.__module__.rpartition('.')
         mod_name = mod_name.replace('_', ' ')
-        return ' '.join([mod_name, cls.__name__])
+        return ' '.join([mod_name, cls.__name__])  # type: ignore
 
-    def __str__(self):
+    def __str__(self) -> str:
         keyring_class = type(self)
         return "{}.{} (priority: {:g})".format(
             keyring_class.__module__, keyring_class.__name__, keyring_class.priority
         )
 
     @abc.abstractmethod
-    def get_password(self, service: str, username: str) -> Optional[str]:
+    def get_password(self, service: str, username: str) -> typing.Optional[str]:
         """Get password of the username for the service"""
         return None
 
@@ -124,8 +127,8 @@ class KeyringBackend(metaclass=KeyringBackendMeta):
     def get_credential(
         self,
         service: str,
-        username: Optional[str],
-    ) -> Optional[credentials.Credential]:
+        username: typing.Optional[str],
+    ) -> typing.Optional[credentials.Credential]:
         """Gets the username and password for the service.
         Returns a Credential instance.
 
@@ -140,18 +143,25 @@ class KeyringBackend(metaclass=KeyringBackendMeta):
                 return credentials.SimpleCredential(username, password)
         return None
 
-    def set_properties_from_env(self):
+    def set_properties_from_env(self) -> None:
         """For all KEYRING_PROPERTY_* env var, set that property."""
 
-        def parse(item):
+        def parse(item: typing.Tuple[str, str]):
             key, value = item
             pre, sep, name = key.partition('KEYRING_PROPERTY_')
             return sep and (name.lower(), value)
 
-        props = filter(None, map(parse, os.environ.items()))
+        props: filter[typing.Tuple[str, str]] = filter(
+            None, map(parse, os.environ.items())
+        )
         for name, value in props:
             setattr(self, name, value)
 
+    def with_properties(self, **kwargs: typing.Any) -> KeyringBackend:
+        alt = copy.copy(self)
+        vars(alt).update(kwargs)
+        return alt
+
 
 class Crypter:
     """Base class providing encryption and decryption"""
@@ -177,7 +187,7 @@ class NullCrypter(Crypter):
         return value
 
 
-def _load_plugins():
+def _load_plugins() -> None:
     """
     Locate all setuptools entry points by the name 'keyring backends'
     and initialize them.
@@ -204,7 +214,7 @@ def _load_plugins():
 
 
 @util.once
-def get_all_keyring():
+def get_all_keyring() -> typing.List[KeyringBackend]:
     """
     Return a list of all implemented keyrings that can be constructed without
     parameters.
@@ -213,3 +223,43 @@ def get_all_keyring():
     viable_classes = KeyringBackend.get_viable_backends()
     rings = util.suppress_exceptions(viable_classes, exceptions=TypeError)
     return list(rings)
+
+
+class SchemeSelectable:
+    """
+    Allow a backend to select different "schemes" for the
+    username and service.
+
+    >>> backend = SchemeSelectable()
+    >>> backend._query('contoso', 'alice')
+    {'username': 'alice', 'service': 'contoso'}
+    >>> backend._query('contoso')
+    {'service': 'contoso'}
+    >>> backend.scheme = 'KeePassXC'
+    >>> backend._query('contoso', 'alice')
+    {'UserName': 'alice', 'Title': 'contoso'}
+    >>> backend._query('contoso', 'alice', foo='bar')
+    {'UserName': 'alice', 'Title': 'contoso', 'foo': 'bar'}
+    """
+
+    scheme = 'default'
+    schemes = dict(
+        default=dict(username='username', service='service'),
+        KeePassXC=dict(username='UserName', service='Title'),
+    )
+
+    def _query(
+        self, service: str, username: typing.Optional[str] = None, **base: typing.Any
+    ) -> typing.Dict[str, str]:
+        scheme = self.schemes[self.scheme]
+        return dict(
+            {
+                scheme['username']: username,
+                scheme['service']: service,
+            }
+            if username is not None
+            else {
+                scheme['service']: service,
+            },
+            **base,
+        )
