@@ -1,4 +1,6 @@
 from ..backend import KeyringBackend
+from .._compat import properties
+from ..errors import NoKeyringError
 
 
 class Keyring(KeyringBackend):
@@ -9,10 +11,12 @@ class Keyring(KeyringBackend):
     >>> kr.get_password('svc', 'user')
     Traceback (most recent call last):
     ...
-    RuntimeError: ...No recommended backend...
+    keyring.errors.NoKeyringError: ...No recommended backend...
     """
 
-    priority = 0
+    @properties.classproperty
+    def priority(cls) -> float:
+        return 0
 
     def get_password(self, service, username, password=None):
         msg = (
@@ -21,6 +25,6 @@ class Keyring(KeyringBackend):
             "you want to use the non-recommended backends. See "
             "https://pypi.org/project/keyring for details."
         )
-        raise RuntimeError(msg)
+        raise NoKeyringError(msg)
 
-    set_password = delete_password = get_password
+    set_password = delete_password = get_password  # type: ignore
