@@ -17,7 +17,7 @@ import getpass
 from . import get_password, delete_password, set_password
 
 
-class PasswordMgr(object):
+class PasswordMgr:
     def get_username(self, realm, authuri):
         return getpass.getuser()
 
@@ -29,8 +29,7 @@ class PasswordMgr(object):
         user = self.get_username(realm, authuri)
         password = get_password(realm, user)
         if password is None:
-            prompt = 'password for %(user)s@%(realm)s for '\
-                '%(authuri)s: ' % vars()
+            prompt = 'password for %(user)s@%(realm)s for ' '%(authuri)s: ' % vars()
             password = getpass.getpass(prompt)
             set_password(realm, user, password)
         return user, password
